#!/usr/bin/env python3
"""
fastuv 命令行接口

提供 fastuv install 和 fastuv uninstall 命令
"""

import sys
import argparse
from typing import Optional

from .installer import UVInstaller
from .uninstaller import UVUninstaller


def create_parser() -> argparse.ArgumentParser:
    """创建命令行参数解析器"""
    parser = argparse.ArgumentParser(
        prog="fastuv",
        description="快速 uv 安装器 - 预配置国内镜像的 uv 安装工具"
    )

    subparsers = parser.add_subparsers(dest="command", help="可用命令")

    # install 命令
    install_parser = subparsers.add_parser("install", help="安装 uv 并配置国内镜像")
    install_parser.add_argument(
        "--version",
        help="指定 uv 版本 (默认: latest)"
    )
    install_parser.add_argument(
        "--proxy",
        default="https://ghfast.top",
        help="下载代理 (默认: https://ghfast.top)"
    )
    install_parser.add_argument(
        "--mirror",
        default="https://pypi.tuna.tsinghua.edu.cn/simple/",
        help="PyPI 镜像源 (默认: 清华源)"
    )
    install_parser.add_argument(
        "--no-hooks",
        action="store_true",
        help="不安装 conda 环境联动 hooks"
    )

    # uninstall 命令
    uninstall_parser = subparsers.add_parser("uninstall", help="卸载 uv 和相关配置")
    uninstall_parser.add_argument(
        "--all",
        action="store_true",
        help="删除所有配置文件和缓存"
    )

    return parser


def cmd_install(args) -> int:
    """处理安装命令"""
    try:
        installer = UVInstaller(
            download_proxy=args.proxy,
            pypi_mirror=args.mirror,
            version=args.version
        )

        print("[开始] 开始安装 uv...")

        # 执行安装
        success = installer.install()

        if success:
            # 安装 conda hooks（除非用户明确禁止）
            if not args.no_hooks:
                print("[配置] 配置 conda 环境联动...")
                installer.install_conda_hooks()

            print("[完成] uv 安装完成！")
            print("[提示] 请运行以下命令或重启终端以使用 uv:")
            print("   source ~/.bashrc  # 或 ~/.zshrc")
            return 0
        else:
            print("[错误] uv 安装失败")
            return 1

    except Exception as e:
        print(f"[错误] 安装过程中发生错误: {e}")
        return 1


def cmd_uninstall(args) -> int:
    """处理卸载命令"""
    try:
        uninstaller = UVUninstaller()

        print("[开始] 开始卸载 uv...")

        success = uninstaller.uninstall(remove_all=args.all)

        if success:
            print("[完成] uv 卸载完成！")
            if args.all:
                print("[清理] 所有配置文件和缓存已清理")
            return 0
        else:
            print("[错误] uv 卸载失败")
            return 1

    except Exception as e:
        print(f"[错误] 卸载过程中发生错误: {e}")
        return 1


def main() -> int:
    """主入口函数"""
    parser = create_parser()

    # 如果没有参数，显示帮助
    if len(sys.argv) == 1:
        parser.print_help()
        return 0

    args = parser.parse_args()

    if args.command == "install":
        return cmd_install(args)
    elif args.command == "uninstall":
        return cmd_uninstall(args)
    else:
        parser.print_help()
        return 1


if __name__ == "__main__":
    sys.exit(main())


"""
fastuv 安装器核心模块

负责下载、安装和配置 uv，包括镜像配置和环境设置
"""

import os
import sys
import platform
import subprocess
import tempfile
import shutil
from pathlib import Path
from typing import Optional
import urllib.request
import json
import importlib.resources


class UVInstaller:
    """uv 安装器类"""

    def __init__(
        self,
        download_proxy: str = "https://ghfast.top",
        pypi_mirror: str = "https://pypi.tuna.tsinghua.edu.cn/simple/",
        version: Optional[str] = None
    ):
        self.download_proxy = download_proxy
        self.pypi_mirror = pypi_mirror
        self.version = version or "latest"
        self.platform = platform.system().lower()
        self.arch = platform.machine().lower()

        # 用户目录路径
        self.home_dir = Path.home()
        self.local_bin_dir = self.home_dir / ".local" / "bin"
        self.config_dir = self.home_dir / ".config" / "uv"

        # uv 相关路径
        self.uv_bin_path = self.local_bin_dir / "uv"
        self.uv_config_path = self.config_dir / "uv.toml"

    def get_latest_uv_version(self) -> str:
        """获取 uv 最新版本号"""
        try:
            url = "https://api.github.com/repos/astral-sh/uv/releases/latest"
            with urllib.request.urlopen(url) as response:
                data = json.loads(response.read().decode())
                return data["tag_name"].lstrip("v")
        except Exception:
            # 如果获取失败，返回默认版本
            return "0.5.0"

    def get_uv_download_url(self) -> str:
        """获取 uv 下载链接"""
        version = self.version if self.version != "latest" else self.get_latest_uv_version()

        # 确定平台和架构
        if self.platform == "linux":
            if self.arch in ["x86_64", "amd64"]:
                archive_name = f"uv-{version}-x86_64-unknown-linux-gnu.tar.gz"
            elif self.arch in ["aarch64", "arm64"]:
                archive_name = f"uv-{version}-aarch64-unknown-linux-gnu.tar.gz"
            else:
                raise RuntimeError(f"Unsupported architecture: {self.arch}")
        elif self.platform == "darwin":
            if self.arch in ["x86_64", "amd64"]:
                archive_name = f"uv-{version}-x86_64-apple-darwin.tar.gz"
            elif self.arch in ["aarch64", "arm64"]:
                archive_name = f"uv-{version}-aarch64-apple-darwin.tar.gz"
            else:
                raise RuntimeError(f"Unsupported architecture: {self.arch}")
        elif self.platform == "windows":
            if self.arch in ["x86_64", "amd64"]:
                archive_name = f"uv-{version}-x86_64-pc-windows-msvc.zip"
            elif self.arch in ["aarch64", "arm64"]:
                archive_name = f"uv-{version}-aarch64-pc-windows-msvc.zip"
            else:
                raise RuntimeError(f"Unsupported architecture: {self.arch}")
        else:
            raise RuntimeError(f"Unsupported platform: {self.platform}")

        # 使用代理下载
        base_url = f"{self.download_proxy}/https://github.com/astral-sh/uv/releases/download/{version}"
        return f"{base_url}/{archive_name}"

    def download_uv_installer_script(self) -> str:
        """下载官方 uv 安装脚本"""
        try:
            # 根据平台选择安装脚本
            if self.platform in ["linux", "darwin"]:
                script_url = "https://github.com/astral-sh/uv/releases/latest/download/uv-installer.sh"
                suffix = '.sh'
            elif self.platform == "windows":
                script_url = "https://github.com/astral-sh/uv/releases/latest/download/uv-installer.ps1"
                suffix = '.ps1'
            else:
                raise RuntimeError(f"Unsupported platform for installer script: {self.platform}")

            # 使用代理
            if self.platform == "windows":
                proxied_url = f"{self.download_proxy}/https://github.com/astral-sh/uv/releases/latest/download/uv-installer.ps1"
            else:
                proxied_url = f"{self.download_proxy}/https://github.com/astral-sh/uv/releases/latest/download/uv-installer.sh"

            # 下载脚本
            with tempfile.NamedTemporaryFile(mode='w+', delete=False, suffix=suffix) as f:
                print(f"正在下载 uv 安装脚本...")
                urllib.request.urlretrieve(proxied_url, f.name)
                return f.name

        except Exception as e:
            raise RuntimeError(f"下载 uv 安装脚本失败: {e}")

    def get_template_content(self, template_name: str) -> str:
        """获取包内模板文件内容"""
        try:
            if hasattr(importlib.resources, 'files'):
                # Python 3.9+
                return (importlib.resources.files('fastuv.templates') / template_name).read_text(encoding='utf-8')
            else:
                # Python 3.8
                with importlib.resources.path('fastuv.templates', template_name) as p:
                    return p.read_text(encoding='utf-8')
        except Exception as e:
            raise RuntimeError(f"无法读取模板文件 {template_name}: {e}")

    def get_hook_content(self, hook_name: str) -> str:
        """获取包内hook文件内容"""
        try:
            if hasattr(importlib.resources, 'files'):
                # Python 3.9+
                return (importlib.resources.files('fastuv.hooks') / hook_name).read_text(encoding='utf-8')
            else:
                # Python 3.8
                with importlib.resources.path('fastuv.hooks', hook_name) as p:
                    return p.read_text(encoding='utf-8')
        except Exception as e:
            raise RuntimeError(f"无法读取hook文件 {hook_name}: {e}")

    def create_custom_installer_script(self, original_script_path: str) -> str:
        """创建自定义安装脚本，注入镜像配置"""

        # 获取包内模板文件内容
        try:
            mirror_config_template = self.get_template_content('shell_injection.sh')
            # 替换模板中的占位符
            mirror_config = mirror_config_template.replace(
                '__DOWNLOAD_PROXY__', self.download_proxy
            ).replace(
                '__PYPI_MIRROR__', self.pypi_mirror
            )
        except Exception as e:
            # 如果无法读取模板文件，使用内置配置
            mirror_config = f'''
# --- fastuv 自定义配置：添加默认 PyPI 和 Python 下载镜像 ---
say "正在配置默认的 PyPI 和 Python 下载镜像..."
local _uv_config_dir="${{XDG_CONFIG_HOME:-$HOME/.config}}/uv"
ensure mkdir -p "$_uv_config_dir"
_UV_DOWNLOAD_PROXY_URL_FOR_CONFIG="${{UV_DOWNLOAD_PROXY:-{self.download_proxy}}}"
_UV_PYPI_MIRROR_URL="${{UV_PYPI_MIRROR:-{self.pypi_mirror}}}"
_UV_PYTHON_INSTALL_MIRROR_URL="${{_UV_DOWNLOAD_PROXY_URL_FOR_CONFIG}}/https://github.com/astral-sh/python-build-standalone/releases/download"
printf "python-install-mirror = \\"%s\\"\\\\n\\\\n[[index]]\\\\nurl = \\"%s\\"\\\\ndefault = true\\\\n" "$_UV_PYTHON_INSTALL_MIRROR_URL" "$_UV_PYPI_MIRROR_URL" > "$_uv_config_dir/uv.toml"
say "[完成] 配置完成。镜像设置如下:"
say "   - Python 下载代理: $_UV_DOWNLOAD_PROXY_URL_FOR_CONFIG"
say "   - PyPI 镜像源: $_UV_PYPI_MIRROR_URL"
say "   - 配置文件路径: $_uv_config_dir/uv.toml"
# --- fastuv 配置结束 ---
'''

        # 读取原始脚本
        with open(original_script_path, 'r', encoding='utf-8') as f:
            script_content = f.read()

        # 注入代理配置
        script_content = script_content.replace(
            'INSTALLER_BASE_URL="${UV_INSTALLER_GITHUB_BASE_URL:-https://github.com}"',
            f'INSTALLER_BASE_URL="${{UV_INSTALLER_GITHUB_BASE_URL:-{self.download_proxy}/https://github.com}}"'
        )

        # 在安装完成前注入镜像配置
        if 'say "everything\'s installed!"' in script_content:
            script_content = script_content.replace(
                'say "everything\'s installed!"',
                mirror_config + '\nsay "everything\'s installed!"'
            )

        # 创建自定义脚本
        custom_script_path = original_script_path.replace('.sh', '-custom.sh')
        with open(custom_script_path, 'w', encoding='utf-8') as f:
            f.write(script_content)

        # 设置执行权限
        os.chmod(custom_script_path, 0o755)

        return custom_script_path

    def ensure_local_bin_dir(self):
        """确保 ~/.local/bin 目录存在"""
        self.local_bin_dir.mkdir(parents=True, exist_ok=True)

    def update_path_in_shell(self):
        """更新 shell 配置文件中的 PATH"""
        shell_configs = [
            self.home_dir / ".bashrc",
            self.home_dir / ".zshrc",
            self.home_dir / ".profile"
        ]

        bin_path_str = str(self.local_bin_dir)
        path_entry = f'export PATH="{bin_path_str}:$PATH"'

        for config_file in shell_configs:
            if config_file.exists():
                content = config_file.read_text()
                if bin_path_str not in content:
                    with open(config_file, 'a') as f:
                        f.write(f"\n# fastuv: Add ~/.local/bin to PATH\n{path_entry}\n")

    def install(self) -> bool:
        """执行安装过程"""
        try:
            print(f"[信息] 目标平台: {self.platform}")
            print(f"[信息] uv 版本: {self.version}")

            # 1. 下载官方安装脚本
            script_path = self.download_uv_installer_script()

            try:
                # 2. 创建自定义安装脚本
                custom_script_path = self.create_custom_installer_script(script_path)

                # 3. 确保安装目录存在
                self.ensure_local_bin_dir()

                # 4. 执行安装
                print("正在执行 uv 安装...")
                env = os.environ.copy()
                env.update({
                    "UV_DOWNLOAD_PROXY": self.download_proxy,
                    "UV_PYPI_MIRROR": self.pypi_mirror
                })

                if self.platform == "windows" and custom_script_path.endswith('.ps1'):
                    # Windows PowerShell
                    result = subprocess.run(
                        ["powershell", "-ExecutionPolicy", "Bypass", "-File", custom_script_path],
                        env=env,
                        capture_output=True,
                        text=True
                    )
                else:
                    # Linux/macOS shell
                    result = subprocess.run(
                        [custom_script_path],
                        env=env,
                        capture_output=True,
                        text=True
                    )

                if result.returncode != 0:
                    print(f"安装失败: {result.stderr}")
                    return False

                # 5. 更新 PATH
                self.update_path_in_shell()

                return True

            finally:
                # 清理临时文件
                for path in [script_path, custom_script_path]:
                    if os.path.exists(path):
                        os.unlink(path)

        except Exception as e:
            print(f"安装过程中发生错误: {e}")
            return False

    def install_conda_hooks(self):
        """安装 conda 环境联动 hooks"""
        try:
            print("正在配置 conda 环境联动...")

            # 获取包内 conda hook 内容
            try:
                conda_hook = self.get_hook_content('conda_env.sh')
            except Exception:
                # 如果无法读取模板文件，使用内置配置
                conda_hook = '''
# fastuv conda 环境联动
_sync_fastuv_conda_env() {
  if [ -n "$CONDA_PREFIX" ]; then
    if [ -z "$UV_PROJECT_ENVIRONMENT" ] || [ "$UV_PROJECT_ENVIRONMENT" != "$CONDA_PREFIX" ]; then
      export UV_PROJECT_ENVIRONMENT="$CONDA_PREFIX"
    fi
  else
    if [ -n "$UV_PROJECT_ENVIRONMENT" ]; then
      unset UV_PROJECT_ENVIRONMENT
    fi
  fi
}

# Bash
if [[ -n "$BASH_VERSION" && ! "$PROMPT_COMMAND" =~ _sync_fastuv_conda_env ]]; then
  PROMPT_COMMAND="_sync_fastuv_conda_env;$PROMPT_COMMAND"
fi

# Zsh
if [[ -n "$ZSH_VERSION" && ! " ${precmd_functions[@]} " =~ " _sync_fastuv_conda_env " ]]; then
  precmd_functions+=(_sync_fastuv_conda_env)
fi
'''

            # 写入到 shell 配置文件
            shell_configs = [
                self.home_dir / ".bashrc",
                self.home_dir / ".zshrc"
            ]

            for config_file in shell_configs:
                if config_file.exists():
                    content = config_file.read_text()
                    if "_sync_fastuv_conda_env" not in content:
                        with open(config_file, 'a') as f:
                            f.write(conda_hook)

            print("[完成] conda 环境联动配置完成")

        except Exception as e:
            print(f"[警告] 配置 conda 环境联动失败: {e}")
            # 不影响主安装流程


"""
fastuv 卸载器模块

负责卸载 uv 和清理相关配置文件
"""

import os
import shutil
from pathlib import Path
from typing import List


class UVUninstaller:
    """uv 卸载器类"""

    def __init__(self):
        self.home_dir = Path.home()
        self.local_bin_dir = self.home_dir / ".local" / "bin"
        self.config_dir = self.home_dir / ".config" / "uv"
        self.cache_dir = self.home_dir / ".cache" / "uv"

        # uv 相关路径
        self.uv_bin_path = self.local_bin_dir / "uv"
        self.uv_config_path = self.config_dir / "uv.toml"

    def get_shell_config_files(self) -> List[Path]:
        """获取 shell 配置文件列表"""
        return [
            self.home_dir / ".bashrc",
            self.home_dir / ".bash_profile",
            self.home_dir / ".zshrc",
            self.home_dir / ".profile",
            self.home_dir / ".config/fish/config.fish"
        ]

    def remove_fastuv_from_shell_configs(self):
        """从 shell 配置文件中移除 fastuv 相关配置"""
        shell_configs = self.get_shell_config_files()

        markers = [
            "# fastuv:",
            "# fastuv conda 环境联动",
            "_sync_fastuv_conda_env",
            "UV_PROJECT_ENVIRONMENT"
        ]

        for config_file in shell_configs:
            if not config_file.exists():
                continue

            try:
                # 读取文件内容
                lines = config_file.read_text(encoding='utf-8').split('\n')

                # 过滤掉包含 fastuv 相关标记的行
                filtered_lines = []
                skip_line = False

                for i, line in enumerate(lines):
                    # 检查是否应该跳过这行
                    should_skip = any(marker in line for marker in markers)

                    if should_skip:
                        skip_line = True
                        continue

                    # 如果之前在跳过，并且这行是空的，可能跳过整个块
                    if skip_line and line.strip() == "":
                        skip_line = False
                        continue

                    if not skip_line:
                        filtered_lines.append(line)

                # 写回文件
                config_file.write_text('\n'.join(filtered_lines), encoding='utf-8')
                print(f"[完成] 已清理 {config_file}")

            except Exception as e:
                print(f"[警告] 清理 {config_file} 失败: {e}")

    def remove_uv_binary(self):
        """移除 uv 二进制文件"""
        if self.uv_bin_path.exists():
            try:
                self.uv_bin_path.unlink()
                print(f"[完成] 已删除 {self.uv_bin_path}")
            except Exception as e:
                print(f"[警告] 删除 uv 二进制文件失败: {e}")
        else:
            print("[信息] uv 二进制文件不存在")

    def remove_uv_config(self):
        """移除 uv 配置文件"""
        if self.config_dir.exists():
            try:
                shutil.rmtree(self.config_dir)
                print(f"[完成] 已删除配置目录 {self.config_dir}")
            except Exception as e:
                print(f"[警告] 删除配置目录失败: {e}")
        else:
            print("[信息] uv 配置目录不存在")

    def remove_uv_cache(self):
        """移除 uv 缓存"""
        if self.cache_dir.exists():
            try:
                shutil.rmtree(self.cache_dir)
                print(f"[完成] 已删除缓存目录 {self.cache_dir}")
            except Exception as e:
                print(f"[警告] 删除缓存目录失败: {e}")
        else:
            print("[信息] uv 缓存目录不存在")

    def check_uv_installation(self) -> bool:
        """检查 uv 是否已安装"""
        return self.uv_bin_path.exists() or self.config_dir.exists()

    def uninstall(self, remove_all: bool = False) -> bool:
        """执行卸载过程"""
        try:
            print("🗑️ 开始卸载 uv...")

            if not self.check_uv_installation():
                print("[信息] 未检测到 uv 安装")
                return True

            # 1. 移除 uv 二进制文件
            self.remove_uv_binary()

            # 2. 移除配置文件
            self.remove_uv_config()

            # 3. 清理 shell 配置中的 fastuv 设置
            print("🧹 清理 shell 配置...")
            self.remove_fastuv_from_shell_configs()

            # 4. 可选：清理缓存
            if remove_all:
                print("🧹 清理缓存文件...")
                self.remove_uv_cache()

            print("[完成] uv 卸载完成！")
            print("💡 请重启终端或运行 'source ~/.bashrc' 以应用更改")

            return True

        except Exception as e:
            print(f"[错误] 卸载过程中发生错误: {e}")
            return False


"""
fastuv 工具模块

提供通用的工具函数
"""

import os
import sys
import platform
import subprocess
import tempfile
from pathlib import Path
from typing import Optional, Dict, Any


def detect_platform() -> Dict[str, str]:
    """检测平台信息"""
    return {
        "system": platform.system().lower(),
        "machine": platform.machine().lower(),
        "platform": platform.platform(),
        "python_version": platform.python_version()
    }


def detect_shell() -> Optional[str]:
    """检测当前使用的 shell"""
    shell = os.environ.get("SHELL", "")
    if "bash" in shell:
        return "bash"
    elif "zsh" in shell:
        return "zsh"
    elif "fish" in shell:
        return "fish"
    else:
        return None


def get_user_config_dir() -> Path:
    """获取用户配置目录"""
    if platform.system() == "Windows":
        return Path(os.environ.get("APPDATA", Path.home() / "AppData" / "Roaming"))
    else:
        return Path.home() / ".config"


def get_user_data_dir() -> Path:
    """获取用户数据目录"""
    if platform.system() == "Windows":
        return Path(os.environ.get("LOCALAPPDATA", Path.home() / "AppData" / "Local"))
    else:
        return Path.home() / ".local" / "share"


def get_user_cache_dir() -> Path:
    """获取用户缓存目录"""
    if platform.system() == "Windows":
        return Path(os.environ.get("TEMP", tempfile.gettempdir()))
    else:
        return Path.home() / ".cache"


def ensure_directory(path: Path) -> bool:
    """确保目录存在"""
    try:
        path.mkdir(parents=True, exist_ok=True)
        return True
    except Exception:
        return False


def is_command_available(command: str) -> bool:
    """检查命令是否可用"""
    try:
        subprocess.run(
            [command, "--version"],
            capture_output=True,
            check=True
        )
        return True
    except (subprocess.CalledProcessError, FileNotFoundError):
        return False


def run_command(
    command: list,
    cwd: Optional[Path] = None,
    env: Optional[Dict[str, str]] = None,
    capture_output: bool = True
) -> subprocess.CompletedProcess:
    """运行命令"""
    try:
        return subprocess.run(
            command,
            cwd=cwd,
            env=env,
            capture_output=capture_output,
            text=True,
            check=True
        )
    except subprocess.CalledProcessError as e:
        return e


def download_file(url: str, dest: Path) -> bool:
    """下载文件"""
    try:
        import urllib.request
        urllib.request.urlretrieve(url, dest)
        return True
    except Exception:
        return False


def read_file_safe(path: Path) -> Optional[str]:
    """安全读取文件"""
    try:
        if path.exists():
            return path.read_text(encoding='utf-8')
        return None
    except Exception:
        return None


def write_file_safe(path: Path, content: str) -> bool:
    """安全写入文件"""
    try:
        ensure_directory(path.parent)
        path.write_text(content, encoding='utf-8')
        return True
    except Exception:
        return False


def format_size(size_bytes: int) -> str:
    """格式化文件大小"""
    if size_bytes == 0:
        return "0B"

    size_names = ["B", "KB", "MB", "GB", "TB"]
    i = 0
    while size_bytes >= 1024 and i < len(size_names) - 1:
        size_bytes /= 1024.0
        i += 1

    return f"{size_bytes:.1f}{size_names[i]}"


def print_success(message: str):
    """打印成功消息"""
    print(f"[完成] {message}")


def print_error(message: str):
    """打印错误消息"""
    print(f"[错误] {message}")


def print_warning(message: str):
    """打印警告消息"""
    print(f"[警告] {message}")


def print_info(message: str):
    """打印信息消息"""
    print(f"[信息] {message}")


def confirm(message: str, default: bool = False) -> bool:
    """用户确认"""
    suffix = " [Y/n]" if default else " [y/N]"

    try:
        response = input(f"{message}{suffix}: ").strip().lower()

        if not response:
            return default

        return response in ["y", "yes", "是", "确认"]
    except (KeyboardInterrupt, EOFError):
        return False